/*******************************************************************************

  Copyright (C) 2010, 2011 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-connector implementation.

    DTV-connector is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-connector is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-connector.

    DTV-connector es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-connector se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/
#pragma once

#include "../../types.h"
#include <boost/thread.hpp>
#include <map>

namespace util {
	class Buffer;

	namespace pool {
		template<typename T> class CircularPool;
	}
}

namespace tuner {
namespace demuxer {
namespace ts {

class Filter;

/**
 * Esta clase realiza el demultiplexado de un ts de archivo, sacando información del @b pcr,
 * chequeando @b continuity @b counter y pasando las secciones a los filtros.
 */
class Demuxer {
public:
	explicit Demuxer( int bufCount=100 );
	virtual ~Demuxer();

	//	Initialization
	void start();
	void stop();

	//	Filters
	bool startFilter( Filter *filter );
	void stopFilter( Filter *filter );

	//	Enqueue bytes to process
	void pushData( util::Buffer *buf );

	//	Buffer handling
	util::Buffer *allocBuffer();
	void freeBuffer( util::Buffer *buf );

protected:
	//  Types
	typedef std::map<ID,Filter *> MapOfFilters;

	//	Parser
	void parse( util::Buffer *buf );
	size_t parse( util::Buffer *buf, size_t parsed );
	Filter *checkContinuity( ID pid, bool tsStart, util::BYTE actualContinuityCounter );
	void tsParser();	

private:
	util::pool::CircularPool<util::Buffer *> *_pool;	
	util::Buffer *_local;
	MapOfFilters  _filters;

	boost::mutex  _mutex;
	boost::thread _thread;
};

}
}
}

