/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../streamplayer.h"
#include <boost/thread.hpp>
#include <boost/date_time/posix_time/posix_time_types.hpp>
#include <list>

//#define DEBUG_CAPTION

namespace tuner {

namespace stream {
	class Pipe;
	class BasicPipe;
}

namespace arib {
	class CaptionData;
	typedef CaptionData Statement;
	class Management;
	class CaptionDemuxer;
}

namespace player {

namespace pt = boost::posix_time;

class CaptionViewer;

class CaptionPlayer : public StreamPlayerImpl<StreamPlayer> {
public:
	CaptionPlayer( Player *player, CaptionViewer *view );
	virtual ~CaptionPlayer();

protected:
	//	Stream player methods
	virtual bool initialize();
	virtual void finalize();
	virtual bool start( const Stream *sInfo );
	virtual void stop( const Stream *sInfo );
	virtual int first() const;	
	virtual bool supportOff() const;

	//	Aux ARIB parsing
	void processUnit( util::BYTE *data, int len );
	int processControlCode( util::BYTE *data, int len );
	int processCSI( util::BYTE *data, int len );
	int processChar( util::BYTE *data, int len );

	//	Commands
	void setCharacterSmallSize();
	void setCharacterMiddleSize();
	void setCharacterNormalSize();
	void setActivePosition( int p1, int p2 );
	void setActivePositionForward();
	void setActivePositionBackward();
	void setActivePositionDown();
	void setActivePositionUp();
	void setColour( int p1 );
	void setForegroundColour( util::BYTE r, util::BYTE g, util::BYTE b, util::BYTE alpha=255 );
	void setBackgroundColour( util::BYTE r, util::BYTE g, util::BYTE b, util::BYTE alpha=255 );

	//	CSI commands
	void setWritingFormat( int *params, int count );
	void setWritingForm( int form );
	void setDisplayPosition( int horizontal, int vertical );
	void setDisplayFormat( int horizontal, int vertical );
	void setCharacterComposition( int horizontal, int vertical );
	void setHorizontalSpacing( int value );
	void setVerticalSpacing( int value );
	void setRasterColour( int value );

	//	Aux demuxer
	void onManagement( tuner::arib::CaptionDemuxer *demux, tuner::arib::Management *mgt );
	void onStatement( tuner::arib::Statement *st );
	void playCaptions();
	bool wait( const pt::ptime &start );
	void cleanData();

private:
	CaptionViewer *_view;
	tuner::stream::BasicPipe *_pipe;
	boost::mutex _mutex;
	boost::thread _thread;
	boost::condition_variable _cWakeup;
	bool _exit;
	bool _running;
	std::string _data;
	std::vector<std::string> _lines;
	std::list<tuner::arib::Statement *> _sts;
#ifdef DEBUG_CAPTION
	std::string _debugData;
#endif
};

}
}

