#pragma once

#include "types.h"
#include "../extension.h"
#include <boost/function.hpp>
#include <vector>

namespace tuner {
namespace player {

class Player;
class StreamInfo;
class StreamPlayer;

class Extension : public tuner::Extension {
public:
	explicit Extension( Player *player );
	virtual ~Extension();

	//	Setup extension
	void addPlayer( StreamPlayer *player );

	//	Attach to a particual service
	void attach( ID srvID );

	//	Stream methods
	ID pcrPID() const;
	
	int nextAudio() { return nextStream( pes::type::audio ); }
	int audioCount() const { return streamCount( pes::type::audio ); }
	const Stream *getAudio( int id=-1 ) const { return get( id, pes::type::audio ); }
	
	int nextVideo() { return nextStream( pes::type::video ); }
	int videoCount() const  { return streamCount( pes::type::video ); }
	const Stream *getVideo( int id=-1 ) const { return get( id, pes::type::video ); }	
	
	int nextSubtitle() { return nextStream( pes::type::subtitle ); }
	int subtitleCount() const { return streamCount( pes::type::subtitle ); }
	const Stream *getSubtitle( int id=-1 ) const { return get( id, pes::type::subtitle ); }

	//	Services notifications
	virtual void onServiceStarted( Service *srv, bool started );

protected:
	typedef std::map<pes::type::type,StreamPlayer *> Players;

	//	Services notifications
	virtual bool onInitialize();
	virtual void onFinalize();
	
	//	Getters
	StreamPlayer *getPlayer( pes::type::type pesType ) const;

	//	Aux
	void start( Service *srv );
	void stop( Service *srv );
	int nextStream( pes::type::type pesType );
	int streamCount( pes::type::type pesType ) const;
	const Stream *get( int id, pes::type::type pesType ) const;

private:
	Player *_player;
	Players _players;
	ID _srvID;
	ID _pcrPID;
	bool _needPCRFilter;
};

}
}

