#pragma once

#include "types.h"
#include <boost/function.hpp>

namespace util {
	class Url;
}

namespace tuner {

namespace stream {
	class Pipe;
}

class ServiceManager;

namespace player {

class Extension;
class StreamPlayer;

class Player {
public:
	Player();	
	virtual ~Player();

	//	Initialization
	virtual bool initialize();
	virtual void finalize();

	//	Url
	typedef boost::function<void (bool)> OnStartedCallback;
	void onStarted( const OnStartedCallback &callback );
	virtual const std::string url() const;

	//	Extension
	Extension *create();
	Extension *extension() const;

	//	Start/Stop notifications
	void start();
	void stop();
	
	//	Streams
	virtual bool startStream( pes::type::type pesType, ID pid, ID type );
	virtual bool startStream( pes::FilterParams *params );	
	virtual bool startPCR( bool filterActive, ID pid );
	virtual void stopStream( pes::type::type pesType, ID pid );
	virtual void stopPCR( bool filterActive, ID pid );		

protected:
	//	Setup extension
	void setupExtension();
	virtual void addAudioPlayer();
	virtual void addVideoPlayer();
	virtual void addCCPlayer();
	virtual StreamPlayer *create( pes::type::type pesType );
	virtual pes::mode::type mode() const;

	//	Aux start/stop methods
	virtual void started( bool value );

	//	Aux URL
	void makeURL( const std::string &type, util::Url *url );

	//	Getters
	ServiceManager *srvMgr() const;
	
private:
	Extension *_ext;
	OnStartedCallback _onStarted;
};

}
}
	
