#pragma once

#include "types.h"
#include <vector>

namespace tuner {
namespace player {

class Extension;
class Player;
class StreamInfo;

class StreamPlayer {
public:
	explicit StreamPlayer( pes::type::type type );	
	virtual ~StreamPlayer();

	//	Setup
	void addStreams( const std::vector<StreamInfo *> &vInfo );
	void addStream( StreamInfo *info );
	
	//	Initialization
	virtual bool initialize();
	virtual void finalize();

	//	Start
	void start();
	void stop( bool clean );

	//	Methods
	int next();
	int count() const;
	const Stream *get( int id=-1 ) const;
	pes::type::type type() const;
	bool collectStream( const ElementaryInfo &info );

protected:
	virtual bool start( const Stream *sInfo );
	virtual void stop( const Stream *sInfo );
	virtual int first() const;	
	virtual bool supportOff() const;
	void start( int id );	

	StreamInfo *getStreamInfo( const ElementaryInfo &info ) const;
	const Stream *getStream( int id ) const;
	int next( int pos, int factor, int size, bool supportOff ) const;

	//	Getter
	virtual Player *player() const=0;
	
private:
	pes::type::type _pesType;
	int _current;
	std::vector<StreamInfo *> _supported;
	std::vector<Stream> _streams;
};

template<class T>
class StreamPlayerImpl : public T {
public:
	StreamPlayerImpl( Player *player, pes::type::type type ) : T(type), _player(player) {}
	virtual ~StreamPlayerImpl() {}

protected:
	virtual Player *player() const { return _player; }

private:
	Player *_player;
};

}
}
	
