#pragma once

#include "../player.h"
#include <boost/thread.hpp>
#include <map>

namespace util {
	class Buffer;
}

namespace tuner {

namespace stream {
	class BasicPipe;
}

namespace player {
namespace ts {

class Output;

class Player : public player::Player {
public:
	explicit Player( Output *out );
	virtual ~Player();

	//	Initialization
	virtual bool initialize();
	virtual void finalize();	

	//	Getters
	virtual const std::string url() const;

	//	Streams
	virtual bool startStream( pes::type::type pesType, ID pid, ID type );
	virtual bool startPCR( bool filterActive, ID pid );

protected:
	typedef std::map<ID,BYTE> CountinuityCounterMap;
	
	virtual void started( bool value );

	void reader();
	void makePAT();
	void makePMT( ID pcrPID );

	void addStream( BYTE *ptr, int &off, const Stream *s );
	void beginPacket( BYTE *ptr, int &off, ID pid );
	void endPacket( BYTE *ptr, int &off );
	void updateCC( BYTE *ptr );
	void checkCC( BYTE *ptr, int len );	
	
private:
	util::Buffer *_pat;
	util::Buffer *_pmt;
	ID _tsID;
	ID _srvID;
	ID _srvPID;

	CountinuityCounterMap _cc;	

	//	Muxer thread
	stream::BasicPipe *_pipe;
	Output *_out;
	boost::thread _thread;
};

}
}
}
	
