/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "../plugin/service.h"
#include <vector>

namespace zapper {
namespace audio {

class Mixer;

/**
 * Clase que provee funcionalidad para la manipulación del audio del reproductor de canales.
 */
class Service : public plugin::ServiceImpl<Service> {
public:
	Service();
	explicit Service( Mixer *mixer );
	virtual ~Service();

	Mixer *createMixer();

	//	Master audio control
	bool isMuted() const;
	bool mute( bool needMute );
	bool toggleMute();

	Volume volumeUp();
	Volume volumeDown();
	Volume volume( Volume vol );
	Volume volume() const;

	void setAudioChannel( audio::channel::type channel );
	audio::channel::type getCurrentAudioChannel() const;
	std::vector<audio::channel::type> supportedAudioChannels() const;
	
	/**
	 * @return Canal de audio por defecto del servicio.
	 */
	audio::channel::type defaultAudioChannel() const;

	//	Standby
	virtual void standby();
	virtual void wakeup();

protected:
	//	Service virtual methods
	virtual void onAttach();	
	virtual bool onOnline();
	virtual void onOffline();
	virtual void onLoadConfig();
	virtual void onResetConfig();

	//	Aux volume
	bool setMute( bool needMute, bool save );
	Volume setVolume( Volume vol );
	void loadMute();

private:
	Mixer *_mixer;
	Volume _volume;
	Volume _oldVolume;
	bool _isMuted;
	audio::channel::type _channels;
};

}
}

