/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include <mpegparser/demuxer/psi/psi.h>
#include <mpegparser/service/serviceid.h>

#define MAX_EIT_TABLES (PSI_TID_EIT_SBL_ACTUAL-PSI_TID_EIT_SBF_ACTUAL+1)

namespace util {
	class Buffer;
}

namespace zapper {
namespace channel {

/**
 * Representa un canal.
 */
class Channel {
public:
	Channel();
	explicit Channel( tuner::Service *srv );
	virtual ~Channel();

	//	Attach to channel manager
	void attach( ChannelManager *mgr );
	void update( Channel *ch );

	//	Channel ID
	int channelID() const;
	void channelID( int channelID );
	
	//	Channel information
	const tuner::ServiceID &id() const;
	size_t network() const;
	const std::string &name() const;
	const std::string channel() const;
	tuner::service::Type type() const;

	//	Shows
	void process( tuner::Eit *eit );
	Show *getCurrentShow() const;
	void getShowsBetween( Shows &shows, const bpt::time_period &period ) const;
	
	//	Channel parental control
	bool isProtected() const;
	parental::age::type parentalAge() const;
	util::BYTE parentalContent() const;

	//	User data
	void toggleFavorite();
	bool isFavorite() const;
	void toggleBlocked();
	void unblock();
	bool isBlocked() const;
	bool isOneSeg() const;

	//	Debug
	void show() const;

	template<class Storage>
	void serialize( Storage &storage ) {
		storage.serialize( "channelID", _channelID );
		storage.serialize( "network", _network );
		storage.serialize( "nitID", _id.nitID );
		storage.serialize( "tsID", _id.tsID );
		storage.serialize( "programID", _id.programID );
		storage.serialize( "name", _name );
		storage.serialize( "type", _type );
		storage.serialize( "channel", _channel );
		storage.serialize( "parentalAge", _parentalAge );
		storage.serialize( "parentalContent", _parentalContent );
		storage.serialize( "isFavorite", _isFavorite );
		storage.serialize( "isBlocked", _isBlocked );
	}

protected:
	bool isProtected( Show *show ) const;
	void addShow( Shows &shows, Show *show, const bpt::time_period &period ) const;

private:
	size_t _network; // Index for custom network (freq/url,etc)
	tuner::ServiceID _id;
	std::string _name;
	tuner::service::Type _type;
	std::string _channel;
	bool _isFavorite;
	bool _isBlocked;
	parental::age::type _parentalAge;
	util::BYTE _parentalContent;
	ChannelManager *_mgr;
	int _channelID;
	Show *_present;
	Show *_following;
	Shows _shows[MAX_EIT_TABLES];
};

struct AllChannelsIterator {
	bool operator()( Channel * /*prog*/ ) const {
		return true;
	}
};

struct FavoriteIterator {
	bool operator()( Channel *prog ) const {
		return prog->isFavorite();
	}
};

struct WithoutBlockedIterator {
	bool operator()( Channel *prog ) const {
		return !prog->isBlocked();
	}
};

struct WithoutProtectedIterator {
	bool operator()( Channel *prog ) const {
		return !prog->isProtected();
	}
};

}
}

