/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "../plugin/service.h"
#include <boost/signals2.hpp>
#include <math.h>

namespace zapper {

namespace control {
	class Service;
}

namespace channel {

class Channel;
typedef std::vector<Channel *> Channels;
typedef boost::signals2::signal<void (Channel *)> OnNewChannelSignal;
typedef boost::signals2::signal<void (Channel *)> OnChannelUpdatedSignal;
typedef boost::signals2::signal<void (Channel *,bool)> OnShowUpdatedSignal;
typedef boost::signals2::signal<void (Channel *ch,bool all)> OnChannelRemovedSignal;

/**
 * Clase que brinda funcionalidad para el manejo de canales.
 */
class ChannelManager : public plugin::ServiceImpl<ChannelManager> {
public:
	explicit ChannelManager( bool useEPG );
	virtual ~ChannelManager();

	//	Channel modification
	OnNewChannelSignal &onNewChannel();
	OnChannelRemovedSignal &onChannelRemoved();
	OnChannelUpdatedSignal &onChannelUpdated();
	OnShowUpdatedSignal &onShowUpdated();
	bool remove( int chID );
	void removeAll();
	void unblockAll();

	//	Iterators
	bool showOneSeg() const;
	void showOneSeg( bool show );
	bool toggleOneSeg();
	int nextChannel( int first, int factor ) const;
	int nextFavorite( int first, int factor ) const;
	template<typename O>
	int nextChannel( int first, int factor, const O &oper ) const;

	//	Getters
	bool haveChannels() const;
	int channelCount() const;
	void getAll( Channels &channels ) const;
	Channel *get( int channelID ) const;

	//	Tuner/Extension notifications (don't use)
	void onChannel( Channel *ch, bool inScan );
	void onUpdateChannel( Channel *ch, bool needPersist );
	void onUpdateShow( Channel *ch, bool current );
	void onBeginScan();
	bool isProtected( bool isBlocked, util::BYTE pContent, parental::age::type pAge ) const;
	void onTimeValid( bool isValid );

protected:
	//	Service virtual methods
	virtual void onAttach();
	virtual bool onOnline();
	virtual void onOffline();
	virtual bool onStart();
	virtual void onStop();
	virtual void onLoadConfig();
	virtual void onResetConfig();

	//	Aux channels
	void erase( Channel *ch, bool removeAll );
	bool isVisible( Channel *ch ) const;
	int getIndex( int chID ) const;
	int getID( Channel *ch ) const;

	//	Aux EPG
	void processEit( const boost::shared_ptr<tuner::Eit> &eit );

	//	Aux control
	void onControlEnabled( bool isEnabled );

private:
	bool _withEPG;
	int _channelID;
	bool _showOneSeg;
	Channels _channels;
	OnNewChannelSignal _onNewChannel;
	OnChannelRemovedSignal _onChannelRemoved;
	OnChannelUpdatedSignal _onChannelUpdated;
	OnShowUpdatedSignal _onShowUpdated;
	control::Service *_control;
	boost::signals2::connection _timeConnection;
};

struct AllChannelsIterator;
struct FavoriteIterator;
struct WithoutBlockedIterator;

template<typename O>
inline int ChannelManager::nextChannel( int first, int factor, const O &oper ) const {
	int result = INVALID_CHANNEL;
	size_t max = _channels.size();
	if (max) {
		//	Get index of first
		int firstIndex=getIndex(first);
		for (size_t p=0; p<max; p++) {
			int a = (firstIndex + factor);
			firstIndex = (int)(a - floor(a/(double)max)*max);

			//	Channel is valid && is visible and oper?
			Channel *ch=_channels[firstIndex];
			if (isVisible(ch) && oper(ch)) {
				result=getID(ch);
				break;
			}
		}
	}
	return result;
}

}
}

