/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../player/types.h"
#include "../audio/types.h"
#include "../display/display.h"
#include <mpegparser/types.h>
#include <boost/signals2.hpp>
#include <vector>

namespace tuner {
namespace player {
	class Extension;
	class Player;
}
}

namespace canvas {
	class System;
}

namespace zapper {

class Tuner;

namespace plugin {
	class PluginManager;
}

namespace player {
	class MediaPlayer;
}

namespace control {
	class Service;
}

namespace channel {

class Channel;
class ChannelManager;

namespace bs2 = boost::signals2;

typedef bs2::signal<void (bool blocked)> OnChannelBlocked;
typedef bs2::signal<void (int, Channel *)> OnChangeChannelSignal;

class ChannelPlayer {
public:
	ChannelPlayer( const std::string &tunerID );
	virtual ~ChannelPlayer();

	//	Change channel
	OnChangeChannelSignal &onChangeChannel();
	OnChannelBlocked &onChannelBlocked();
	int change( int channelID );
	int current() const;

	//	Player methods
	bool isBlocked() const;
	int nextVideo();
	int videoCount() const;
	std::string videoInfo( int id=-1 ) const;

	int nextAudio();
	int audioCount() const;
	std::string audioInfo( int id=-1 ) const;

	int nextSubtitle();
	int subtitleCount() const;
	std::string subtitleInfo( int id=-1 ) const;

	void setFullScreen();
	void resize( int x, int y, int w, int h );
	void mute( bool needMute );
	void volume( audio::Volume vol );
	audio::channel::type getAudio() const;
	display::aspect::type getAspect() const;
	display::mode::type getResolution() const;

	//	Plugin manager notifications
	void onAttach( std::vector<std::string> &deps );
	bool onOnline( plugin::PluginManager *mgr, canvas::System *sys );
	void onOffline( plugin::PluginManager *mgr );
	bool onStart( plugin::PluginManager *mgr );
	void onStop( plugin::PluginManager *mgr );
	void onStandby( plugin::PluginManager *mgr );
	void onWakeup( plugin::PluginManager *mgr );

protected:
	void stopCurrent();
	void startPlayer( bool start );
	void restartPlayer();
	void onRemoved( Channel *ch, bool removeAll );
	void onUpdated( Channel *ch );
	void onShowUpdated( Channel *ch, bool current );
	void onSessionChanged( bool state );
	void block( bool value );
	tuner::player::Extension *playerExtension() const;

private:
	std::string _tunerID;
	tuner::player::Player *_tsPlayer;
	Tuner *_tuner;
	Channel *_ch;
	ChannelManager *_chs;
	control::Service *_control;
	player::MediaPlayer *_mp;
	OnChangeChannelSignal _onChange;
	OnChannelBlocked _onBlocked;
	bool _blocked;
};

}
}
