/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../plugin/service.h"
#include "../parental.h"
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/signals2.hpp>

namespace zapper {
namespace control {

namespace pt = boost::posix_time;

typedef boost::signals2::signal<void (bool)> OnEnabledChanged;
typedef boost::signals2::signal<void (bool)> OnSessionChanged;

/**
 * Esta clase tiene como finalidad configurar los parámetros de control parental para que luego los canales y programas 
 * exhibidos se ajusten a ellos.
 */
class Service : public plugin::ServiceImpl<Service> {
public:
	Service();
	virtual ~Service();

	//	Authentication
	bool enabled() const;
	OnEnabledChanged &onEnabledChanged();

	//	Session handling
	bool sessionIsOpen() const;
	OnSessionChanged &onSessionChanged();

	//	Set/Check PIN
	bool checkPIN( const std::string &pass );
	void setPIN( const std::string &pass );
	void setTimeExpiration( int expireInSeconds );
	void expire();

	//	Parental control
	util::BYTE parentalContent() const;
	void setParentalContent( util::BYTE );
	parental::age::type parentalAge() const;
	void setParentalAge( parental::age::type age );

	//	Standby
	virtual void standby();

protected:
	//	Service virtual methods
	virtual void onLoadConfig();
	virtual void onResetConfig();

	void enableSession();
	bool isActive() const;

private:
	std::string _pass;
	util::BYTE _content;
	parental::age::type _age;
	bool _sessionActive;
	int _sessionExpireInSeconds;
	pt::ptime _lastValidation;
	OnEnabledChanged _enabledChanged;
	OnSessionChanged _sessionChanged;
};

}
}
