/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

namespace zapper {
namespace display {

#define ZAPPER_LAYER_ZINDEX       300
#define ZAPPER_FIRST_TRANSPARENCY 155
#define ZAPPER_MIN_TRANSPARENCY   0
#define ZAPPER_MAX_TRANSPARENCY   10.0

namespace layer {

#define LAYER_LIST(fnc) \
	fnc(video) \
	fnc(middleware) \
	fnc(zapper)

#define DO_ENUM_LAYER(n) n,
enum type {
	LAYER_LIST(DO_ENUM_LAYER)
	LAST_LAYER
};
#undef DO_ENUM_LAYER

}	//	namespace layer

namespace scan {

enum type {
	unknown,
	p,
	i
};

}	//	namespace scan

typedef struct {
	const char *name;
	const char *mode;
	int width;
	int height;
	int frequency;
	display::scan::type type;
} Mode;

typedef struct {
	const char *name;
	int x;
	int y;
} Aspect;

namespace aspect {

#define ASPECT_RATIO_LIST(fnc)					\
	fnc(1_1,1,1)								\
	fnc(4_3,4,3)								\
	fnc(14_9,14,9)								\
	fnc(16_9,16,9)

#define DO_ENUM_ASPECT(n,w,h) a##n,
enum type {
	unknown,
	automatic,
	ASPECT_RATIO_LIST(DO_ENUM_ASPECT)
	LAST_ASPECT
};
#undef DO_ENUM_ASPECT

}	//	namespace aspect

namespace mode {

#define MODE_LIST(fnc)							\
	fnc(240p_60,320,240,60,p)					\
	fnc(480p_60,640,480,60,p)					\
	fnc(480p_72,640,480,72,p)					\
	fnc(480p_75,640,480,75,p)					\
	fnc(480p_85,640,480,85,p)					\
	fnc(480i_60,640,480,60,i)					\
	fnc(480i_72,640,480,72,i)					\
	fnc(480i_75,640,480,75,i)					\
	fnc(480i_85,640,480,85,i)					\
	fnc(800_600_60,800,600,60,p)				\
	fnc(800_600_72,800,600,72,p)				\
	fnc(800_600_75,800,600,75,p)				\
	fnc(800_600_85,800,600,85,p)				\
	fnc(1024_768_60,1024,768,60,p)				\
	fnc(1024_768_72,1024,768,72,p)				\
	fnc(1024_768_75,1024,768,75,p)				\
	fnc(1024_768_85,1024,768,85,p)				\
	fnc(576p_50,720,576,50,p)					\
	fnc(576i_50,720,576,50,i)					\
	fnc(NTSC,720,480,60,i)						\
	fnc(720p_50,1280,720,50,p)					\
	fnc(720p_60,1280,720,60,p)					\
	fnc(720i_50,1280,720,50,i)					\
	fnc(720i_60,1280,720,60,i)					\
	fnc(1080p_24,1920,1080,24,p)				\
	fnc(1080p_25,1920,1080,25,p)				\
	fnc(1080p_30,1920,1080,30,p)				\
	fnc(1080p_50,1920,1080,50,p)				\
	fnc(1080p_60,1920,1080,60,p)				\
	fnc(1080i_25,1920,1080,25,i)				\
	fnc(1080i_30,1920,1080,30,i)				\
	fnc(1080i_50,1920,1080,50,i)				\
	fnc(1080i_60,1920,1080,60,i)

#define DO_ENUM_MODES(n,w,h,f,t) m##n,
enum type {
	unknown,
	MODE_LIST(DO_ENUM_MODES)
	LAST_MODE
};
#undef DO_ENUM_MODES

}	//	namespace mode

namespace connector {

#define CONNECTOR_LIST(fnc)	  \
	fnc(vga,VGA) \
	fnc(composite,Video Compuesto) \
	fnc(svideo,S-Video) \
	fnc(component,Por Componentes) \
	fnc(dvi,DVI) \
	fnc(hdmi,HDMI)	

#define DO_ENUM_CONNECTOR(n,i) n,
enum type {
	unknown,
	CONNECTOR_LIST(DO_ENUM_CONNECTOR)
	LAST_CONNECTOR
};
#undef DO_ENUM_CONNECTOR

}	//	namespace connector

namespace modulator {

#define MODULATOR_LIST(fnc)						\
	fnc(Ninguno,None)							\
	fnc(Canal 3,CH3)							\
	fnc(Canal 4,CH4)

#define DO_ENUM_MODULATOR(n,c) m##c,
enum type {
	unknown,
	MODULATOR_LIST(DO_ENUM_MODULATOR)
	LAST_MODULATOR
};
#undef DO_ENUM_MODULATOR

}	//	namespace modulator

//	Returns the connector (tv out) name
const char *getConnectorName( connector::type conn );
//	Returns the mode
display::Mode &getMode( display::mode::type vMode );
//	Returns the aspect mode
display::Aspect &getAspect( display::aspect::type vAspect );
//	Returns the modulator mode
const char *getModulatorName( display::modulator::type mod );

}	//	namespace display
}	//	namespace zapper
