/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "display.h"
#include "../plugin/service.h"
#include <boost/signals2.hpp>
#include <vector>

namespace canvas {
	class System;
}

namespace zapper {
namespace display {

class Layer;

/**
 * Clase que provee funcionalidad para la manipulación de las opciones de visualización del reproductor de canales.
 */
class DisplayService : public plugin::ServiceImpl<DisplayService> {
public:
	explicit DisplayService( canvas::System *sys );
	virtual ~DisplayService();

	//	Connectors
	virtual std::vector<connector::type> supportedConnectors() const;
	connector::type getActiveConnector() const;
	
	//	Video modes
	virtual std::vector<mode::type> supportedModes( connector::type conn ) const;
	bool setMode( connector::type conn, mode::type vMode, bool doActive=true );
	mode::type getCurrentVideoMode( connector::type conn ) const;
	typedef boost::signals2::signal<void (mode::type)> ModeChangedSignal;
	ModeChangedSignal &onModeChanged();

	//	Video aspect ratio
	typedef boost::signals2::signal<void (aspect::type)> AspectChangedSignal;
	virtual std::vector<aspect::type> supportedAspects() const;
	bool setAspect( aspect::type vAspect );
	aspect::type getCurrentAspect() const;
	AspectChangedSignal &onAspectChanged();

	//	Video modulator
	virtual std::vector<modulator::type> supportedModulators() const;
	bool setModulator( modulator::type mod );
	modulator::type getCurrentModulator() const;

	//	Transparency
	bool setTransparency( int value );
	int getTransparency() const;

	//	Layers
	Layer *getLayer( layer::type layer );

protected:
	//	Service virtual methods
	virtual bool onOnline();
	virtual void onOffline();
	virtual void onLoadConfig();
	virtual void onResetConfig();

	//	Connector aux
	bool setActive( connector::type conn );
	virtual connector::type getDefaultActiveConnector() const;	
	bool connectorIsValid( connector::type conn ) const ;
	
	//	Video mode aux
	virtual bool supportMultipleModes() const;
	virtual mode::type defaultVideoMode( connector::type conn ) const;
	virtual bool setVideoMode( connector::type conn, mode::type vMode, bool doActive );

	//	Video aspect aux
	virtual aspect::type defaultVideoAspect() const;
	virtual bool setVideoAspect( aspect::type vAspect );

	//	Video modulator aux
	virtual modulator::type defaultVideoModulator() const;
	virtual bool setVideoModulator( modulator::type mod );

	//	Transparency aux
	virtual bool setDisplayTransparency( int value );
	virtual int defaultTransparency() const;

	//	Layer aux
	virtual Layer *createLayer( layer::type layerID ) const;
	virtual bool setupLayers();
	bool createLayers();
	void destroyLayers();

private:
	canvas::System *_sys;
	int _transparency;
	connector::type _connector;
	mode::type _mode[connector::LAST_CONNECTOR];
	ModeChangedSignal _onModeChanged;
	aspect::type _aspect;
	AspectChangedSignal _onAspectChanged;
	modulator::type _modulator;
	std::vector<Layer *> _layers;
};

}
}

