/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "input.h"
#include "../plugin/service.h"
#include <map>

#define LOW_KEY_PRIORITY  1
#define MED_KEY_PRIORITY  100
#define HI_KEY_PRIORITY   1000
#define MAX_KEY_PRIORITY  10000

namespace zapper {
namespace input {

/**
 * Clase que se encarga de proveer funcionalidad para el despacho de eventos de teclado.
 */
class InputService : public plugin::ServiceImpl<InputService> {
public:
	InputService( void );
	virtual ~InputService();

	//	Methods
	bool dispatchKey( util::key::type key, bool state );
	void reserveKeys( const std::string &id, const Callback &callback, const Keys &keys, int priority );
	void renice( const std::string &id, int priority );

	void dispatchButton( canvas::event::ButtonEvent *evt );
	void reserveButtons( const ButtonCallback &callback );

protected:
	//	Types
	class ButtonListener;
	class Listener;
	typedef std::vector<Listener *> Listeners;
	typedef std::map<util::key::type,Listener *> Dispatched;	

	//	State to handle listener keys
	void pushListener( Listener *l, util::key::type key );
	Listener *popListener( util::key::type key );

private:
	Listeners _listeners;
	Dispatched _dispatched;
	ButtonCallback _buttonCallback;
};

}
}

