/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../display/display.h"
#include <mpegparser/service/extension/application/controller/applicationcontroller.h>
#include <boost/signals2.hpp>
#include <vector>

namespace tuner {
	namespace app {
		class Spawner;
		class ApplicationExtension;
	}
}

namespace zapper {

namespace plugin {
	class PluginManager;
}

namespace display {
	class Layer;
}

namespace channel {
	class ChannelService;
}

namespace input {
	class InputService;
}

namespace middleware {

class ApplicationService;
typedef std::pair<tuner::app::video::mode::type,display::mode::type> AppResolution;

class ApplicationController : public tuner::app::ApplicationController {
public:
	ApplicationController( int maxProcessInParallel );
	virtual ~ApplicationController();

	//	Zapper initialization
	virtual void init( plugin::PluginManager *pluginMgr, display::Layer *layer );
	virtual void fin();

	//	Zapper methods
	void setResolution( display::mode::type mode );
	void setAspect( display::aspect::type mode );
	void disableApplications( bool disable );

	//	Application notifications
	virtual void onApplicationAdded( tuner::app::Application *app );
	virtual void onApplicationRemoved( tuner::app::Application *app );
	virtual void onApplicationChanged( tuner::app::Application *app );
	virtual void onDownloadProgress( tuner::app::Application *app, int step, int total );	

	//	Application operations
	virtual void reserveKeys( const tuner::app::ApplicationID &id, const tuner::app::Keys &keys, int priority );
	virtual void resizeVideo( float x, float y, float w, float h );

	//	Spawner
	virtual tuner::app::Spawner *spawner();

protected:
	//	Types
	typedef struct {
		float x, y, w, h;
	} VideoResolution;

	//	Virtual method
	virtual void setupFactories( tuner::app::ApplicationExtension *ext );
	virtual tuner::app::video::mode::type setupLayer( const std::vector<tuner::app::video::mode::type> &modes );
	virtual void enableLayer( bool layer );

	//	Channel blocked
	void onChannelBlocked( bool blocked );
	void disableMask( util::DWORD mask, bool set );

	//	Aux resolution
	display::Layer *layer() const;
	AppResolution calculateResolution( const std::vector<tuner::app::video::mode::type> &modes );
	void doSetAspect( display::aspect::type mode );
	void updateVideo();

private:
	tuner::app::Spawner *_spawner;
	AppResolution _appResolution;
	VideoResolution _videoResolution;
	display::mode::type _tvMode;
	display::aspect::type _tvAspect;
	display::Layer *_layer;

	ApplicationService *_srv;
	input::InputService *_input;
	channel::ChannelService	*_player;

	boost::signals2::connection _cChannelBlocked;
	util::DWORD _disableMask;
};

}
}

