/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../input/input.h"
#include "../display/display.h"
#include "../plugin/service.h"
#include <mpegparser/service/extension/application/applicationid.h>
#include <boost/signals2.hpp>

namespace tuner {
namespace app {
	class Application;
}
}

namespace zapper {
namespace middleware {

class ApplicationInfo;
class ApplicationController;
typedef boost::signals2::signal<void (bool isEnabled)> OnLayerEnabled;
typedef boost::signals2::signal<void (const tuner::app::ApplicationID &id, int step, int total)> OnDownloadProgress;

/**
 * Clase que contiene a las aplicaciones y provee funcionalidad para activar y desactivar aplicaciones.
 */
class ApplicationService : public plugin::ServiceImpl<ApplicationService> {
public:
	ApplicationService( const std::string &tuner, ApplicationController *ctrl );
	virtual ~ApplicationService();

	//	Public methods
	static const std::string getMiddlewareVersion();
	const std::vector<ApplicationInfo *> &getApplications() const;
	void start( const std::string &id );
	void disableApplications( bool needDisable );
	
	//	Notifications
	OnDownloadProgress &onDownloadProgress();	
	OnLayerEnabled &onLayerEnabled();
	
	//	Application life cycle (implementation)
	void addApplication( ApplicationInfo *info );
	void rmApplication( const tuner::app::ApplicationID &id );
	void chgApplication( const tuner::app::ApplicationID &id, tuner::app::status::type cur );
	void onDownloadProgress( const tuner::app::ApplicationID &id, int step, int total );
	void onLayerChanged( bool isLayerEnabled );	

protected:
	//	Service virtual methods
	virtual void onAttach();
	virtual bool onStart();
	virtual void onStop();

	//	Display notifications
	void onDisplayModeChanged( display::mode::type dMode );
	void onDisplayAspectChanged( display::aspect::type dAspect );

	//	Mount notifications
	void onMountChanged( const std::string &mountPoint, bool isMounted );

private:
	std::string _tuner;
	std::vector<ApplicationInfo *> _apps;
	ApplicationController *_ctrl;
	boost::signals2::connection _onMountChanged, _onModeChanged, _onAspectChanged;
	OnLayerEnabled _onLayerEnabled;
	OnDownloadProgress _onDownloadProgress;
};

}
}

