/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "../audio/types.h"
#include "../display/display.h"
#include <util/types.h>
#include <string>
#include <vector>
#include <stack>

namespace zapper {
namespace player {

class MediaPlayer;

class Player {
public:
	explicit Player();
	virtual ~Player();

	//	Initialization
	bool initialize();
	void finalize();

	//	MediaPlayer methods
	MediaPlayer *create();
	void destroy( MediaPlayer *player );

	//	Display notifications
	void setResolution( int width, int height );
	const Resolution &getDisplayResolution() const;
	void setAspectRatio( display::aspect::type vAspect );
	display::aspect::type getDisplayAspect() const;

	//	Single player implementation
	void stackPlayer( MediaPlayer *mp );
	void popPlayer();

protected:
	typedef std::pair<MediaPlayer *, std::string> StackedPlayer;
	
	//	Media player
	virtual MediaPlayer *createMediaPlayer() const=0;
	virtual bool supportMultiplePlayers() const;

	//	Display notifications
	virtual void onDisplayChanged();

private:
	Resolution _dResolution;
	display::aspect::type _dAspect;
	std::vector<MediaPlayer *> _players;
	MediaPlayer *_singlePlayer;
	MediaPlayer *_currentActive;
	std::stack<StackedPlayer> _activePlayers;
	bool _needPopPlayer;
};

}
}

