/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include <string>

namespace zapper {
namespace plugin {

class PluginManager;

/**
 * Clase que brinda la interfaz para la implementación de @em plugins.
 */
class Plugin {
public:
	Plugin( const std::string &name );
	virtual ~Plugin( void );

	//	Initialization
	void initialize( PluginManager *mgr );
	void finalize( void );
	bool isInitialized( void ) const;

	//	Enable
	bool isEnabled( void ) const;
	void enable( bool newState );

	//	Reset config
	void resetConfig( void );

	//	Getters
	const std::string &name() const;

protected:
	//	Aux initialization
	virtual bool onInitialize( void );
	virtual void onFinalize( void );

	//	Aux enable state
	virtual bool defaultEnabledState( void ) const;	
	virtual void onEnable( bool isEnabled );

	//	Aux config
	virtual void onResetConfig( void );

	PluginManager *mgr() const;
	std::string pluginName( void ) const;

private:
	std::string _name;
	PluginManager *_mgr;
	bool _isEnabled;
};

}
}

