/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include <boost/function.hpp>
#include <typeinfo>
#include <string>

namespace zapper {
namespace plugin {

class PluginManager;
class DependencyHandler;
class Dependency;

/**
 * Superclase de todos los servicios como @c Tuner o @c ChannelManager brinda la funcionalidad común para iniciar y 
 * detener servicios.
 */
class Service {
public:
	Service();
	Service( const std::string &name );
	virtual ~Service();

	//	Start/stop
	bool isStarted() const;	
	void start();
	void stop();
	virtual void onStopFired();

	//	Standby
	virtual void standby();
	virtual void wakeup();

	//	Add/release use reference
	void addRef();
	void release();
	bool haveReferences() const;

	//	Add dependency
	void addDependency( Dependency *dep );
	void addSimpleDependency( const std::string &name );
	void addOptionalDependency( const std::string &name );	

	//	Config
	void resetConfig();

	//	Registration status
	bool isOnline() const;
	
	//	Getters
	PluginManager *mgr() const;
	const std::string &name() const;
	virtual const std::string className() const;

	//	PluginManager methods
	void attach( PluginManager *mgr );
	void dettach();
	virtual const std::type_info &type() const=0;
	template<typename T> T *get();

	//	PluginManager notifications
	void onServiceChanged( Service *srv );

protected:
	//	Aux attach
	virtual void onAttach();
	virtual void onDettach();

	//	Aux online
	virtual bool onOnline();
	virtual void onOffline();

	//	Aux start/stop
	void tryStart();
	virtual bool onStart();
	virtual void onStop();

	//	Aux config
	virtual void onResetConfig();
	virtual void onLoadConfig();

	//	Aux status
	void status( service::status::registration st );
	void status( service::status::run st );		
	void checkStatus();	

	//	Getters
	void setName( const std::string &name );
	void setName( const std::string &name, int id );

private:
	std::string _name;
	service::status::registration _registration;
	service::status::run _run;
	int _references;
	PluginManager *_mgr;
	DependencyHandler *_dependencies;
};

template<typename T> T *Service::get() {
	return dynamic_cast<T *>(this);
}

//	TODO: Remove this class
template<typename T>
class ServiceImpl : public Service {
public:
	ServiceImpl() {}
	ServiceImpl( const std::string &name ) : Service( name ) {}
	virtual ~ServiceImpl() {}

	virtual const std::type_info &type() const {
		return typeid(T);
	}
};

}
}
