/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "../plugin/service.h"
#include <mpegparser/tuner/types.h>
#include <vector>

namespace tuner {
	class Tuner;
	class Provider;
	class Extension;
	class Service;

	namespace stream {
		class Pipe;
	}
}

namespace zapper {

namespace channel {
	class Channel;
}

class TunerListener;

namespace impl {
	class ScanIterator;
}

/**
 * Clase responsable de sintonizar canales y obtener la información sobre ellos.
 */
class Tuner : public plugin::ServiceImpl<Tuner> {
public:
	Tuner( int id, tuner::Provider *provider, const std::string &ramDisk );
	virtual ~Tuner();

	//  Listener
	void addListener( TunerListener *listener );

	//	Attach extension into service manager
	void attach( tuner::Extension *ext );
	void detach( tuner::Extension *ext );

	//	Tune channel
	bool startService( size_t tunerID, tuner::ID srvID );
	void stopService( tuner::ID srvID );

	//	Scan
	bool isScanning() const;
	void startScan();
	void stopScan();

	//	State
	tuner::state::type state();
	tuner::signal::type signal();

	virtual const std::string className() const;
	virtual void onStopFired();
	virtual void standby();

	static void attach( plugin::PluginManager *mgr, tuner::Extension *ext, const std::string &tuner="Tuner0" );
	static void detach( plugin::PluginManager *mgr, tuner::Extension *ext, const std::string &tuner="Tuner0" );	

protected:
	//  Virtual methods
	virtual bool onOnline();	
	virtual void onOffline();
	virtual bool onStart();
	virtual void onStop();

private:
	impl::ScanIterator *_scanIter;
	tuner::Tuner *_tuner;
};

}
